import cyvcf2
import pyfastx
import himut.util
import himut.vcflib
import numpy as np
import pandas as pd
from plotnine import * 
from collections import defaultdict
from typing import Dict, List, Tuple


purine = set(["A", "G"])
pyrimidine = set(["T", "C"])
sub_lst = ["C>A", "C>G", "C>T", "T>A", "T>C", "T>G"]
purine2pyrimidine = {"A": "T", "T": "A", "G": "C", "C": "G", "N": "N"}

tri_lst = [
    "ACA",
    "ACC",
    "ACG",
    "ACT",
    "ATA",
    "ATC",
    "ATG",
    "ATT",
    "CCA",
    "CCC",
    "CCG",
    "CCT",
    "CTA",
    "CTC",
    "CTG",
    "CTT",
    "GCA",
    "GCC",
    "GCG",
    "GCT",
    "GTA",
    "GTC",
    "GTG",
    "GTT",
    "TCA",
    "TCC",
    "TCG",
    "TCT",
    "TTA",
    "TTC",
    "TTG",
    "TTT"
]

sbs52_lst = [
    "A[C>A]A",
    "A[C>A]C",
    "A[C>A]G",
    "A[C>A]T",
    "C[C>A]A",
    "C[C>A]C",
    "C[C>A]G",
    "C[C>A]T",
    "G[C>A]A",
    "G[C>A]C",
    "G[C>A]G",
    "G[C>A]T",
    "T[C>A]A",
    "T[C>A]C",
    "T[C>A]G",
    "T[C>A]T",
    "A[C>T]A",
    "A[C>T]C",
    "A[C>T]G",
    "A[C>T]T",
    "C[C>T]A",
    "C[C>T]C",
    "C[C>T]G",
    "C[C>T]T",
    "G[C>T]A",
    "G[C>T]C",
    "G[C>T]G",
    "G[C>T]T",
    "T[C>T]A",
    "T[C>T]C",
    "T[C>T]G",
    "T[C>T]T",
    "A[C>G]A",
    "A[C>G]C",
    "A[C>G]G",
    "A[C>G]T",
    "C[C>G]A",
    "C[C>G]C",
    "C[C>G]G",
    "G[C>G]A",
    "G[C>G]C",
    "T[C>G]A",
    "A[T>A]A",
    "A[T>A]C",
    "A[T>A]G",
    "A[T>A]T",
    "C[T>A]A",
    "C[T>A]C",
    "C[T>A]G",
    "G[T>A]A",
    "G[T>A]C",
    "T[T>A]A"
]

sbs96_lst = [
    "A[C>A]A",
    "A[C>A]C",
    "A[C>A]G",
    "A[C>A]T",
    "A[C>G]A",
    "A[C>G]C",
    "A[C>G]G",
    "A[C>G]T",
    "A[C>T]A",
    "A[C>T]C",
    "A[C>T]G",
    "A[C>T]T",
    "A[T>A]A",
    "A[T>A]C",
    "A[T>A]G",
    "A[T>A]T",
    "A[T>C]A",
    "A[T>C]C",
    "A[T>C]G",
    "A[T>C]T",
    "A[T>G]A",
    "A[T>G]C",
    "A[T>G]G",
    "A[T>G]T",
    "C[C>A]A",
    "C[C>A]C",
    "C[C>A]G",
    "C[C>A]T",
    "C[C>G]A",
    "C[C>G]C",
    "C[C>G]G",
    "C[C>G]T",
    "C[C>T]A",
    "C[C>T]C",
    "C[C>T]G",
    "C[C>T]T",
    "C[T>A]A",
    "C[T>A]C",
    "C[T>A]G",
    "C[T>A]T",
    "C[T>C]A",
    "C[T>C]C",
    "C[T>C]G",
    "C[T>C]T",
    "C[T>G]A",
    "C[T>G]C",
    "C[T>G]G",
    "C[T>G]T",
    "G[C>A]A",
    "G[C>A]C",
    "G[C>A]G",
    "G[C>A]T",
    "G[C>G]A",
    "G[C>G]C",
    "G[C>G]G",
    "G[C>G]T",
    "G[C>T]A",
    "G[C>T]C",
    "G[C>T]G",
    "G[C>T]T",
    "G[T>A]A",
    "G[T>A]C",
    "G[T>A]G",
    "G[T>A]T",
    "G[T>C]A",
    "G[T>C]C",
    "G[T>C]G",
    "G[T>C]T",
    "G[T>G]A",
    "G[T>G]C",
    "G[T>G]G",
    "G[T>G]T",
    "T[C>A]A",
    "T[C>A]C",
    "T[C>A]G",
    "T[C>A]T",
    "T[C>G]A",
    "T[C>G]C",
    "T[C>G]G",
    "T[C>G]T",
    "T[C>T]A",
    "T[C>T]C",
    "T[C>T]G",
    "T[C>T]T",
    "T[T>A]A",
    "T[T>A]C",
    "T[T>A]G",
    "T[T>A]T",
    "T[T>C]A",
    "T[T>C]C",
    "T[T>C]G",
    "T[T>C]T",
    "T[T>G]A",
    "T[T>G]C",
    "T[T>G]G",
    "T[T>G]T",
]

sbs1536_lst = [
    "AA[C>A]AA",
    "AA[C>A]AC",
    "AA[C>A]AG",
    "AA[C>A]AT",
    "AA[C>A]CA",
    "AA[C>A]CC",
    "AA[C>A]CG",
    "AA[C>A]CT",
    "AA[C>A]GA",
    "AA[C>A]GC",
    "AA[C>A]GG",
    "AA[C>A]GT",
    "AA[C>A]TA",
    "AA[C>A]TC",
    "AA[C>A]TG",
    "AA[C>A]TT",
    "AC[C>A]AA",
    "AC[C>A]AC",
    "AC[C>A]AG",
    "AC[C>A]AT",
    "AC[C>A]CA",
    "AC[C>A]CC",
    "AC[C>A]CG",
    "AC[C>A]CT",
    "AC[C>A]GA",
    "AC[C>A]GC",
    "AC[C>A]GG",
    "AC[C>A]GT",
    "AC[C>A]TA",
    "AC[C>A]TC",
    "AC[C>A]TG",
    "AC[C>A]TT",
    "AG[C>A]AA",
    "AG[C>A]AC",
    "AG[C>A]AG",
    "AG[C>A]AT",
    "AG[C>A]CA",
    "AG[C>A]CC",
    "AG[C>A]CG",
    "AG[C>A]CT",
    "AG[C>A]GA",
    "AG[C>A]GC",
    "AG[C>A]GG",
    "AG[C>A]GT",
    "AG[C>A]TA",
    "AG[C>A]TC",
    "AG[C>A]TG",
    "AG[C>A]TT",
    "AT[C>A]AA",
    "AT[C>A]AC",
    "AT[C>A]AG",
    "AT[C>A]AT",
    "AT[C>A]CA",
    "AT[C>A]CC",
    "AT[C>A]CG",
    "AT[C>A]CT",
    "AT[C>A]GA",
    "AT[C>A]GC",
    "AT[C>A]GG",
    "AT[C>A]GT",
    "AT[C>A]TA",
    "AT[C>A]TC",
    "AT[C>A]TG",
    "AT[C>A]TT",
    "CA[C>A]AA",
    "CA[C>A]AC",
    "CA[C>A]AG",
    "CA[C>A]AT",
    "CA[C>A]CA",
    "CA[C>A]CC",
    "CA[C>A]CG",
    "CA[C>A]CT",
    "CA[C>A]GA",
    "CA[C>A]GC",
    "CA[C>A]GG",
    "CA[C>A]GT",
    "CA[C>A]TA",
    "CA[C>A]TC",
    "CA[C>A]TG",
    "CA[C>A]TT",
    "CC[C>A]AA",
    "CC[C>A]AC",
    "CC[C>A]AG",
    "CC[C>A]AT",
    "CC[C>A]CA",
    "CC[C>A]CC",
    "CC[C>A]CG",
    "CC[C>A]CT",
    "CC[C>A]GA",
    "CC[C>A]GC",
    "CC[C>A]GG",
    "CC[C>A]GT",
    "CC[C>A]TA",
    "CC[C>A]TC",
    "CC[C>A]TG",
    "CC[C>A]TT",
    "CG[C>A]AA",
    "CG[C>A]AC",
    "CG[C>A]AG",
    "CG[C>A]AT",
    "CG[C>A]CA",
    "CG[C>A]CC",
    "CG[C>A]CG",
    "CG[C>A]CT",
    "CG[C>A]GA",
    "CG[C>A]GC",
    "CG[C>A]GG",
    "CG[C>A]GT",
    "CG[C>A]TA",
    "CG[C>A]TC",
    "CG[C>A]TG",
    "CG[C>A]TT",
    "CT[C>A]AA",
    "CT[C>A]AC",
    "CT[C>A]AG",
    "CT[C>A]AT",
    "CT[C>A]CA",
    "CT[C>A]CC",
    "CT[C>A]CG",
    "CT[C>A]CT",
    "CT[C>A]GA",
    "CT[C>A]GC",
    "CT[C>A]GG",
    "CT[C>A]GT",
    "CT[C>A]TA",
    "CT[C>A]TC",
    "CT[C>A]TG",
    "CT[C>A]TT",
    "GA[C>A]AA",
    "GA[C>A]AC",
    "GA[C>A]AG",
    "GA[C>A]AT",
    "GA[C>A]CA",
    "GA[C>A]CC",
    "GA[C>A]CG",
    "GA[C>A]CT",
    "GA[C>A]GA",
    "GA[C>A]GC",
    "GA[C>A]GG",
    "GA[C>A]GT",
    "GA[C>A]TA",
    "GA[C>A]TC",
    "GA[C>A]TG",
    "GA[C>A]TT",
    "GC[C>A]AA",
    "GC[C>A]AC",
    "GC[C>A]AG",
    "GC[C>A]AT",
    "GC[C>A]CA",
    "GC[C>A]CC",
    "GC[C>A]CG",
    "GC[C>A]CT",
    "GC[C>A]GA",
    "GC[C>A]GC",
    "GC[C>A]GG",
    "GC[C>A]GT", 
    "GC[C>A]TA",
    "GC[C>A]TC",
    "GC[C>A]TG",
    "GC[C>A]TT",
    "GG[C>A]AA",
    "GG[C>A]AC",
    "GG[C>A]AG",
    "GG[C>A]AT",
    "GG[C>A]CA",
    "GG[C>A]CC",
    "GG[C>A]CG",
    "GG[C>A]CT",
    "GG[C>A]GA",
    "GG[C>A]GC",
    "GG[C>A]GG",
    "GG[C>A]GT",
    "GG[C>A]TA",
    "GG[C>A]TC",
    "GG[C>A]TG",
    "GG[C>A]TT",
    "GT[C>A]AA",
    "GT[C>A]AC",
    "GT[C>A]AG",
    "GT[C>A]AT",
    "GT[C>A]CA",
    "GT[C>A]CC",
    "GT[C>A]CG",
    "GT[C>A]CT",
    "GT[C>A]GA",
    "GT[C>A]GC",
    "GT[C>A]GG",
    "GT[C>A]GT",
    "GT[C>A]TA",
    "GT[C>A]TC",
    "GT[C>A]TG",
    "GT[C>A]TT",
    "TA[C>A]AA",
    "TA[C>A]AC",
    "TA[C>A]AG",
    "TA[C>A]AT",
    "TA[C>A]CA",
    "TA[C>A]CC",
    "TA[C>A]CG",
    "TA[C>A]CT",
    "TA[C>A]GA",
    "TA[C>A]GC",
    "TA[C>A]GG",
    "TA[C>A]GT",
    "TA[C>A]TA",
    "TA[C>A]TC",
    "TA[C>A]TG",
    "TA[C>A]TT",
    "TC[C>A]AA",
    "TC[C>A]AC",
    "TC[C>A]AG",
    "TC[C>A]AT",
    "TC[C>A]CA",
    "TC[C>A]CC",
    "TC[C>A]CG",
    "TC[C>A]CT", 
    "TC[C>A]GA",
    "TC[C>A]GC",
    "TC[C>A]GG",
    "TC[C>A]GT",
    "TC[C>A]TA",
    "TC[C>A]TC",
    "TC[C>A]TG",
    "TC[C>A]TT",
    "TG[C>A]AA",
    "TG[C>A]AC",
    "TG[C>A]AG",
    "TG[C>A]AT",
    "TG[C>A]CA",
    "TG[C>A]CC",
    "TG[C>A]CG",
    "TG[C>A]CT",
    "TG[C>A]GA",
    "TG[C>A]GC",
    "TG[C>A]GG",
    "TG[C>A]GT",
    "TG[C>A]TA",
    "TG[C>A]TC",
    "TG[C>A]TG",
    "TG[C>A]TT",
    "TT[C>A]AA",
    "TT[C>A]AC",
    "TT[C>A]AG",
    "TT[C>A]AT",
    "TT[C>A]CA",
    "TT[C>A]CC",
    "TT[C>A]CG",
    "TT[C>A]CT",
    "TT[C>A]GA",
    "TT[C>A]GC",
    "TT[C>A]GG",
    "TT[C>A]GT",
    "TT[C>A]TA",
    "TT[C>A]TC",
    "TT[C>A]TG",
    "TT[C>A]TT",
    "AA[C>G]AA",
    "AA[C>G]AC",
    "AA[C>G]AG",
    "AA[C>G]AT",
    "AA[C>G]CA",
    "AA[C>G]CC",
    "AA[C>G]CG",
    "AA[C>G]CT",
    "AA[C>G]GA",
    "AA[C>G]GC",
    "AA[C>G]GG",
    "AA[C>G]GT",
    "AA[C>G]TA",
    "AA[C>G]TC",
    "AA[C>G]TG",
    "AA[C>G]TT",
    "AC[C>G]AA",
    "AC[C>G]AC",
    "AC[C>G]AG",
    "AC[C>G]AT",
    "AC[C>G]CA",
    "AC[C>G]CC",
    "AC[C>G]CG",
    "AC[C>G]CT",
    "AC[C>G]GA",
    "AC[C>G]GC",
    "AC[C>G]GG",
    "AC[C>G]GT",
    "AC[C>G]TA",
    "AC[C>G]TC",
    "AC[C>G]TG",
    "AC[C>G]TT",
    "AG[C>G]AA",
    "AG[C>G]AC",
    "AG[C>G]AG",
    "AG[C>G]AT",
    "AG[C>G]CA",
    "AG[C>G]CC",
    "AG[C>G]CG",
    "AG[C>G]CT",
    "AG[C>G]GA",
    "AG[C>G]GC",
    "AG[C>G]GG",
    "AG[C>G]GT",
    "AG[C>G]TA",
    "AG[C>G]TC",
    "AG[C>G]TG",
    "AG[C>G]TT",
    "AT[C>G]AA",
    "AT[C>G]AC",
    "AT[C>G]AG",
    "AT[C>G]AT",
    "AT[C>G]CA",
    "AT[C>G]CC",
    "AT[C>G]CG",
    "AT[C>G]CT",
    "AT[C>G]GA",
    "AT[C>G]GC",
    "AT[C>G]GG",
    "AT[C>G]GT", 
    "AT[C>G]TA",
    "AT[C>G]TC",
    "AT[C>G]TG",
    "AT[C>G]TT",
    "CA[C>G]AA",
    "CA[C>G]AC",
    "CA[C>G]AG",
    "CA[C>G]AT",
    "CA[C>G]CA",
    "CA[C>G]CC",
    "CA[C>G]CG",
    "CA[C>G]CT",
    "CA[C>G]GA",
    "CA[C>G]GC",
    "CA[C>G]GG",
    "CA[C>G]GT",
    "CA[C>G]TA",
    "CA[C>G]TC",
    "CA[C>G]TG",
    "CA[C>G]TT",
    "CC[C>G]AA",
    "CC[C>G]AC",
    "CC[C>G]AG",
    "CC[C>G]AT",
    "CC[C>G]CA",
    "CC[C>G]CC",
    "CC[C>G]CG",
    "CC[C>G]CT",
    "CC[C>G]GA",
    "CC[C>G]GC",
    "CC[C>G]GG",
    "CC[C>G]GT",
    "CC[C>G]TA",
    "CC[C>G]TC",
    "CC[C>G]TG",
    "CC[C>G]TT",
    "CG[C>G]AA",
    "CG[C>G]AC",
    "CG[C>G]AG",
    "CG[C>G]AT",
    "CG[C>G]CA",
    "CG[C>G]CC",
    "CG[C>G]CG",
    "CG[C>G]CT",
    "CG[C>G]GA",
    "CG[C>G]GC",
    "CG[C>G]GG",
    "CG[C>G]GT",
    "CG[C>G]TA",
    "CG[C>G]TC",
    "CG[C>G]TG",
    "CG[C>G]TT",
    "CT[C>G]AA",
    "CT[C>G]AC",
    "CT[C>G]AG",
    "CT[C>G]AT",
    "CT[C>G]CA",
    "CT[C>G]CC",
    "CT[C>G]CG",
    "CT[C>G]CT",
    "CT[C>G]GA",
    "CT[C>G]GC",
    "CT[C>G]GG",
    "CT[C>G]GT",
    "CT[C>G]TA",
    "CT[C>G]TC",
    "CT[C>G]TG",
    "CT[C>G]TT",
    "GA[C>G]AA",
    "GA[C>G]AC",
    "GA[C>G]AG",
    "GA[C>G]AT",
    "GA[C>G]CA",
    "GA[C>G]CC",
    "GA[C>G]CG",
    "GA[C>G]CT",
    "GA[C>G]GA",
    "GA[C>G]GC",
    "GA[C>G]GG",
    "GA[C>G]GT",
    "GA[C>G]TA",
    "GA[C>G]TC",
    "GA[C>G]TG",
    "GA[C>G]TT",
    "GC[C>G]AA",
    "GC[C>G]AC",
    "GC[C>G]AG",
    "GC[C>G]AT",
    "GC[C>G]CA",
    "GC[C>G]CC",
    "GC[C>G]CG",
    "GC[C>G]CT",
    "GC[C>G]GA",
    "GC[C>G]GC",
    "GC[C>G]GG",
    "GC[C>G]GT",
    "GC[C>G]TA",
    "GC[C>G]TC",
    "GC[C>G]TG",
    "GC[C>G]TT",
    "GG[C>G]AA",
    "GG[C>G]AC",
    "GG[C>G]AG",
    "GG[C>G]AT",
    "GG[C>G]CA",
    "GG[C>G]CC",
    "GG[C>G]CG",
    "GG[C>G]CT",
    "GG[C>G]GA",
    "GG[C>G]GC",
    "GG[C>G]GG",
    "GG[C>G]GT",
    "GG[C>G]TA",
    "GG[C>G]TC",
    "GG[C>G]TG",
    "GG[C>G]TT",
    "GT[C>G]AA",
    "GT[C>G]AC",
    "GT[C>G]AG",
    "GT[C>G]AT",
    "GT[C>G]CA",
    "GT[C>G]CC",
    "GT[C>G]CG",
    "GT[C>G]CT",
    "GT[C>G]GA",
    "GT[C>G]GC",
    "GT[C>G]GG",
    "GT[C>G]GT",
    "GT[C>G]TA",
    "GT[C>G]TC",
    "GT[C>G]TG",
    "GT[C>G]TT",
    "TA[C>G]AA",
    "TA[C>G]AC",
    "TA[C>G]AG",
    "TA[C>G]AT",
    "TA[C>G]CA",
    "TA[C>G]CC",
    "TA[C>G]CG",
    "TA[C>G]CT",
    "TA[C>G]GA",
    "TA[C>G]GC",
    "TA[C>G]GG",
    "TA[C>G]GT",
    "TA[C>G]TA",
    "TA[C>G]TC",
    "TA[C>G]TG",
    "TA[C>G]TT",
    "TC[C>G]AA",
    "TC[C>G]AC",
    "TC[C>G]AG",
    "TC[C>G]AT",
    "TC[C>G]CA",
    "TC[C>G]CC",
    "TC[C>G]CG",
    "TC[C>G]CT",
    "TC[C>G]GA",
    "TC[C>G]GC",
    "TC[C>G]GG",
    "TC[C>G]GT",
    "TC[C>G]TA",
    "TC[C>G]TC",
    "TC[C>G]TG",
    "TC[C>G]TT",
    "TG[C>G]AA",
    "TG[C>G]AC",
    "TG[C>G]AG",
    "TG[C>G]AT",
    "TG[C>G]CA",
    "TG[C>G]CC",
    "TG[C>G]CG",
    "TG[C>G]CT",
    "TG[C>G]GA",
    "TG[C>G]GC",
    "TG[C>G]GG",
    "TG[C>G]GT",
    "TG[C>G]TA",
    "TG[C>G]TC",
    "TG[C>G]TG",
    "TG[C>G]TT",
    "TT[C>G]AA",
    "TT[C>G]AC",
    "TT[C>G]AG",
    "TT[C>G]AT",
    "TT[C>G]CA",
    "TT[C>G]CC",
    "TT[C>G]CG",
    "TT[C>G]CT",
    "TT[C>G]GA",
    "TT[C>G]GC",
    "TT[C>G]GG",
    "TT[C>G]GT",
    "TT[C>G]TA",
    "TT[C>G]TC",
    "TT[C>G]TG",
    "TT[C>G]TT",
    "AA[C>T]AA",
    "AA[C>T]AC",
    "AA[C>T]AG",
    "AA[C>T]AT",
    "AA[C>T]CA",
    "AA[C>T]CC",
    "AA[C>T]CG",
    "AA[C>T]CT",
    "AA[C>T]GA",
    "AA[C>T]GC",
    "AA[C>T]GG",
    "AA[C>T]GT",
    "AA[C>T]TA",
    "AA[C>T]TC",
    "AA[C>T]TG",
    "AA[C>T]TT",
    "AC[C>T]AA",
    "AC[C>T]AC",
    "AC[C>T]AG",
    "AC[C>T]AT",
    "AC[C>T]CA",
    "AC[C>T]CC",
    "AC[C>T]CG",
    "AC[C>T]CT",
    "AC[C>T]GA",
    "AC[C>T]GC",
    "AC[C>T]GG",
    "AC[C>T]GT",
    "AC[C>T]TA",
    "AC[C>T]TC",
    "AC[C>T]TG",
    "AC[C>T]TT",
    "AG[C>T]AA",
    "AG[C>T]AC",
    "AG[C>T]AG",
    "AG[C>T]AT",
    "AG[C>T]CA",
    "AG[C>T]CC",
    "AG[C>T]CG",
    "AG[C>T]CT",
    "AG[C>T]GA",
    "AG[C>T]GC",
    "AG[C>T]GG",
    "AG[C>T]GT",
    "AG[C>T]TA",
    "AG[C>T]TC",
    "AG[C>T]TG",
    "AG[C>T]TT",
    "AT[C>T]AA",
    "AT[C>T]AC",
    "AT[C>T]AG",
    "AT[C>T]AT",
    "AT[C>T]CA",
    "AT[C>T]CC",
    "AT[C>T]CG",
    "AT[C>T]CT",
    "AT[C>T]GA",
    "AT[C>T]GC",
    "AT[C>T]GG",
    "AT[C>T]GT",
    "AT[C>T]TA",
    "AT[C>T]TC",
    "AT[C>T]TG",
    "AT[C>T]TT",
    "CA[C>T]AA",
    "CA[C>T]AC",
    "CA[C>T]AG",
    "CA[C>T]AT",
    "CA[C>T]CA",
    "CA[C>T]CC",
    "CA[C>T]CG",
    "CA[C>T]CT",
    "CA[C>T]GA",
    "CA[C>T]GC",
    "CA[C>T]GG",
    "CA[C>T]GT",
    "CA[C>T]TA",
    "CA[C>T]TC",
    "CA[C>T]TG",
    "CA[C>T]TT",
    "CC[C>T]AA",
    "CC[C>T]AC",
    "CC[C>T]AG",
    "CC[C>T]AT",
    "CC[C>T]CA",
    "CC[C>T]CC",
    "CC[C>T]CG",
    "CC[C>T]CT",
    "CC[C>T]GA",
    "CC[C>T]GC",
    "CC[C>T]GG",
    "CC[C>T]GT",
    "CC[C>T]TA",
    "CC[C>T]TC",
    "CC[C>T]TG",
    "CC[C>T]TT",
    "CG[C>T]AA",
    "CG[C>T]AC",
    "CG[C>T]AG",
    "CG[C>T]AT",
    "CG[C>T]CA",
    "CG[C>T]CC",
    "CG[C>T]CG",
    "CG[C>T]CT",
    "CG[C>T]GA",
    "CG[C>T]GC",
    "CG[C>T]GG",
    "CG[C>T]GT",
    "CG[C>T]TA",
    "CG[C>T]TC",
    "CG[C>T]TG",
    "CG[C>T]TT",
    "CT[C>T]AA",
    "CT[C>T]AC",
    "CT[C>T]AG",
    "CT[C>T]AT",
    "CT[C>T]CA",
    "CT[C>T]CC",
    "CT[C>T]CG",
    "CT[C>T]CT",    
    "CT[C>T]GA",
    "CT[C>T]GC",
    "CT[C>T]GG",
    "CT[C>T]GT",
    "CT[C>T]TA",
    "CT[C>T]TC",
    "CT[C>T]TG",
    "CT[C>T]TT",
    "GA[C>T]AA",
    "GA[C>T]AC",
    "GA[C>T]AG",
    "GA[C>T]AT",
    "GA[C>T]CA",
    "GA[C>T]CC",
    "GA[C>T]CG",
    "GA[C>T]CT",
    "GA[C>T]GA",
    "GA[C>T]GC",
    "GA[C>T]GG",
    "GA[C>T]GT",
    "GA[C>T]TA",
    "GA[C>T]TC",
    "GA[C>T]TG",
    "GA[C>T]TT",
    "GC[C>T]AA",
    "GC[C>T]AC",
    "GC[C>T]AG",
    "GC[C>T]AT",
    "GC[C>T]CA",
    "GC[C>T]CC",
    "GC[C>T]CG",
    "GC[C>T]CT",
    "GC[C>T]GA",
    "GC[C>T]GC",
    "GC[C>T]GG",
    "GC[C>T]GT",
    "GC[C>T]TA",
    "GC[C>T]TC",
    "GC[C>T]TG",
    "GC[C>T]TT",
    "GG[C>T]AA",
    "GG[C>T]AC",
    "GG[C>T]AG",
    "GG[C>T]AT",
    "GG[C>T]CA",
    "GG[C>T]CC",
    "GG[C>T]CG",
    "GG[C>T]CT",
    "GG[C>T]GA",
    "GG[C>T]GC",
    "GG[C>T]GG",
    "GG[C>T]GT",
    "GG[C>T]TA",
    "GG[C>T]TC",
    "GG[C>T]TG",
    "GG[C>T]TT",
    "GT[C>T]AA",
    "GT[C>T]AC",
    "GT[C>T]AG",
    "GT[C>T]AT",
    "GT[C>T]CA",
    "GT[C>T]CC",
    "GT[C>T]CG",
    "GT[C>T]CT",
    "GT[C>T]GA",
    "GT[C>T]GC",
    "GT[C>T]GG",
    "GT[C>T]GT",
    "GT[C>T]TA",
    "GT[C>T]TC",
    "GT[C>T]TG",
    "GT[C>T]TT",
    "TA[C>T]AA",
    "TA[C>T]AC",
    "TA[C>T]AG",
    "TA[C>T]AT",
    "TA[C>T]CA",
    "TA[C>T]CC",
    "TA[C>T]CG",
    "TA[C>T]CT",
    "TA[C>T]GA",
    "TA[C>T]GC",
    "TA[C>T]GG",
    "TA[C>T]GT",
    "TA[C>T]TA",
    "TA[C>T]TC",
    "TA[C>T]TG",
    "TA[C>T]TT",
    "TC[C>T]AA",
    "TC[C>T]AC",
    "TC[C>T]AG",
    "TC[C>T]AT",
    "TC[C>T]CA",
    "TC[C>T]CC",
    "TC[C>T]CG",
    "TC[C>T]CT",
    "TC[C>T]GA",
    "TC[C>T]GC",
    "TC[C>T]GG",
    "TC[C>T]GT",
    "TC[C>T]TA",
    "TC[C>T]TC",
    "TC[C>T]TG",
    "TC[C>T]TT",
    "TG[C>T]AA",
    "TG[C>T]AC",
    "TG[C>T]AG",
    "TG[C>T]AT",
    "TG[C>T]CA",
    "TG[C>T]CC",
    "TG[C>T]CG",
    "TG[C>T]CT",
    "TG[C>T]GA",
    "TG[C>T]GC",
    "TG[C>T]GG",
    "TG[C>T]GT",
    "TG[C>T]TA",
    "TG[C>T]TC",
    "TG[C>T]TG",
    "TG[C>T]TT",    
    "TT[C>T]AA",
    "TT[C>T]AC",
    "TT[C>T]AG",
    "TT[C>T]AT",
    "TT[C>T]CA",
    "TT[C>T]CC",
    "TT[C>T]CG",
    "TT[C>T]CT",
    "TT[C>T]GA",
    "TT[C>T]GC",
    "TT[C>T]GG",
    "TT[C>T]GT",
    "TT[C>T]TA",
    "TT[C>T]TC",
    "TT[C>T]TG",
    "TT[C>T]TT",
    "AA[T>A]AA",
    "AA[T>A]AC",
    "AA[T>A]AG",
    "AA[T>A]AT",
    "AA[T>A]CA",
    "AA[T>A]CC",
    "AA[T>A]CG",
    "AA[T>A]CT",
    "AA[T>A]GA",
    "AA[T>A]GC",
    "AA[T>A]GG",
    "AA[T>A]GT",
    "AA[T>A]TA",
    "AA[T>A]TC",
    "AA[T>A]TG",
    "AA[T>A]TT",
    "AC[T>A]AA",
    "AC[T>A]AC",
    "AC[T>A]AG",
    "AC[T>A]AT",
    "AC[T>A]CA",
    "AC[T>A]CC",
    "AC[T>A]CG",
    "AC[T>A]CT",
    "AC[T>A]GA",
    "AC[T>A]GC",
    "AC[T>A]GG",
    "AC[T>A]GT",
    "AC[T>A]TA",
    "AC[T>A]TC",
    "AC[T>A]TG",
    "AC[T>A]TT",
    "AG[T>A]AA",
    "AG[T>A]AC",
    "AG[T>A]AG",
    "AG[T>A]AT",
    "AG[T>A]CA",
    "AG[T>A]CC",
    "AG[T>A]CG",
    "AG[T>A]CT",
    "AG[T>A]GA",
    "AG[T>A]GC",
    "AG[T>A]GG",
    "AG[T>A]GT",
    "AG[T>A]TA",
    "AG[T>A]TC",
    "AG[T>A]TG",
    "AG[T>A]TT",
    "AT[T>A]AA",
    "AT[T>A]AC",
    "AT[T>A]AG",
    "AT[T>A]AT",
    "AT[T>A]CA",
    "AT[T>A]CC",
    "AT[T>A]CG",
    "AT[T>A]CT",
    "AT[T>A]GA",
    "AT[T>A]GC",
    "AT[T>A]GG", 
    "AT[T>A]GT",
    "AT[T>A]TA",
    "AT[T>A]TC",
    "AT[T>A]TG",
    "AT[T>A]TT",
    "CA[T>A]AA",
    "CA[T>A]AC",
    "CA[T>A]AG",
    "CA[T>A]AT",
    "CA[T>A]CA",
    "CA[T>A]CC",
    "CA[T>A]CG",
    "CA[T>A]CT",
    "CA[T>A]GA",
    "CA[T>A]GC",
    "CA[T>A]GG",
    "CA[T>A]GT",
    "CA[T>A]TA",
    "CA[T>A]TC",
    "CA[T>A]TG",
    "CA[T>A]TT",
    "CC[T>A]AA",
    "CC[T>A]AC",
    "CC[T>A]AG",
    "CC[T>A]AT",
    "CC[T>A]CA",
    "CC[T>A]CC",
    "CC[T>A]CG",
    "CC[T>A]CT",
    "CC[T>A]GA",
    "CC[T>A]GC",
    "CC[T>A]GG",
    "CC[T>A]GT",
    "CC[T>A]TA",
    "CC[T>A]TC",
    "CC[T>A]TG",
    "CC[T>A]TT",
    "CG[T>A]AA",
    "CG[T>A]AC",
    "CG[T>A]AG",
    "CG[T>A]AT",
    "CG[T>A]CA",
    "CG[T>A]CC",
    "CG[T>A]CG",
    "CG[T>A]CT",
    "CG[T>A]GA",
    "CG[T>A]GC",
    "CG[T>A]GG",
    "CG[T>A]GT",
    "CG[T>A]TA",
    "CG[T>A]TC",
    "CG[T>A]TG",
    "CG[T>A]TT",
    "CT[T>A]AA",
    "CT[T>A]AC",
    "CT[T>A]AG",
    "CT[T>A]AT",
    "CT[T>A]CA",
    "CT[T>A]CC",
    "CT[T>A]CG",
    "CT[T>A]CT",
    "CT[T>A]GA",
    "CT[T>A]GC",
    "CT[T>A]GG",
    "CT[T>A]GT",
    "CT[T>A]TA",
    "CT[T>A]TC",
    "CT[T>A]TG",
    "CT[T>A]TT",
    "GA[T>A]AA",
    "GA[T>A]AC",
    "GA[T>A]AG",
    "GA[T>A]AT",
    "GA[T>A]CA",
    "GA[T>A]CC",
    "GA[T>A]CG",
    "GA[T>A]CT",
    "GA[T>A]GA",
    "GA[T>A]GC",
    "GA[T>A]GG",
    "GA[T>A]GT",
    "GA[T>A]TA",
    "GA[T>A]TC",
    "GA[T>A]TG",
    "GA[T>A]TT",
    "GC[T>A]AA",
    "GC[T>A]AC",
    "GC[T>A]AG",
    "GC[T>A]AT",
    "GC[T>A]CA",
    "GC[T>A]CC",
    "GC[T>A]CG",
    "GC[T>A]CT",
    "GC[T>A]GA",
    "GC[T>A]GC",
    "GC[T>A]GG",
    "GC[T>A]GT",
    "GC[T>A]TA",
    "GC[T>A]TC",
    "GC[T>A]TG",
    "GC[T>A]TT",
    "GG[T>A]AA",
    "GG[T>A]AC",
    "GG[T>A]AG",
    "GG[T>A]AT",
    "GG[T>A]CA",
    "GG[T>A]CC",
    "GG[T>A]CG",
    "GG[T>A]CT",
    "GG[T>A]GA",
    "GG[T>A]GC",
    "GG[T>A]GG",
    "GG[T>A]GT",
    "GG[T>A]TA",
    "GG[T>A]TC",
    "GG[T>A]TG",
    "GG[T>A]TT",
    "GT[T>A]AA",
    "GT[T>A]AC",    
    "GT[T>A]AG",
    "GT[T>A]AT",
    "GT[T>A]CA",
    "GT[T>A]CC",
    "GT[T>A]CG",
    "GT[T>A]CT",
    "GT[T>A]GA",
    "GT[T>A]GC",
    "GT[T>A]GG",
    "GT[T>A]GT",
    "GT[T>A]TA",
    "GT[T>A]TC",
    "GT[T>A]TG",
    "GT[T>A]TT",
    "TA[T>A]AA",
    "TA[T>A]AC",
    "TA[T>A]AG",
    "TA[T>A]AT",
    "TA[T>A]CA",
    "TA[T>A]CC",
    "TA[T>A]CG",
    "TA[T>A]CT",
    "TA[T>A]GA",
    "TA[T>A]GC",
    "TA[T>A]GG",
    "TA[T>A]GT",
    "TA[T>A]TA",
    "TA[T>A]TC",
    "TA[T>A]TG",
    "TA[T>A]TT",
    "TC[T>A]AA",
    "TC[T>A]AC",
    "TC[T>A]AG",
    "TC[T>A]AT",
    "TC[T>A]CA",
    "TC[T>A]CC",
    "TC[T>A]CG",
    "TC[T>A]CT",
    "TC[T>A]GA",
    "TC[T>A]GC",
    "TC[T>A]GG",
    "TC[T>A]GT",
    "TC[T>A]TA",
    "TC[T>A]TC",
    "TC[T>A]TG",
    "TC[T>A]TT",
    "TG[T>A]AA",
    "TG[T>A]AC",
    "TG[T>A]AG",
    "TG[T>A]AT",
    "TG[T>A]CA",
    "TG[T>A]CC",
    "TG[T>A]CG",
    "TG[T>A]CT",
    "TG[T>A]GA",
    "TG[T>A]GC",
    "TG[T>A]GG",
    "TG[T>A]GT",
    "TG[T>A]TA", 
    "TG[T>A]TC",
    "TG[T>A]TG",
    "TG[T>A]TT",
    "TT[T>A]AA",
    "TT[T>A]AC",
    "TT[T>A]AG",
    "TT[T>A]AT",
    "TT[T>A]CA",
    "TT[T>A]CC",
    "TT[T>A]CG",
    "TT[T>A]CT",
    "TT[T>A]GA",
    "TT[T>A]GC",
    "TT[T>A]GG",
    "TT[T>A]GT",
    "TT[T>A]TA",
    "TT[T>A]TC",
    "TT[T>A]TG",
    "TT[T>A]TT",
    "AA[T>C]AA",
    "AA[T>C]AC",
    "AA[T>C]AG",
    "AA[T>C]AT",
    "AA[T>C]CA",
    "AA[T>C]CC",
    "AA[T>C]CG",
    "AA[T>C]CT",
    "AA[T>C]GA",
    "AA[T>C]GC",
    "AA[T>C]GG",
    "AA[T>C]GT",
    "AA[T>C]TA",
    "AA[T>C]TC",
    "AA[T>C]TG",
    "AA[T>C]TT",
    "AC[T>C]AA",
    "AC[T>C]AC",
    "AC[T>C]AG",
    "AC[T>C]AT",
    "AC[T>C]CA",
    "AC[T>C]CC",
    "AC[T>C]CG",
    "AC[T>C]CT",
    "AC[T>C]GA",
    "AC[T>C]GC",
    "AC[T>C]GG",
    "AC[T>C]GT",
    "AC[T>C]TA",
    "AC[T>C]TC",
    "AC[T>C]TG",
    "AC[T>C]TT",
    "AG[T>C]AA",
    "AG[T>C]AC",
    "AG[T>C]AG",
    "AG[T>C]AT",
    "AG[T>C]CA",
    "AG[T>C]CC",
    "AG[T>C]CG",
    "AG[T>C]CT",
    "AG[T>C]GA",
    "AG[T>C]GC",
    "AG[T>C]GG",
    "AG[T>C]GT",
    "AG[T>C]TA",
    "AG[T>C]TC",
    "AG[T>C]TG",
    "AG[T>C]TT",
    "AT[T>C]AA",
    "AT[T>C]AC",
    "AT[T>C]AG",
    "AT[T>C]AT",
    "AT[T>C]CA",
    "AT[T>C]CC",
    "AT[T>C]CG",
    "AT[T>C]CT",
    "AT[T>C]GA",
    "AT[T>C]GC",
    "AT[T>C]GG",
    "AT[T>C]GT",
    "AT[T>C]TA",
    "AT[T>C]TC",
    "AT[T>C]TG",
    "AT[T>C]TT",
    "CA[T>C]AA",
    "CA[T>C]AC",
    "CA[T>C]AG",
    "CA[T>C]AT",
    "CA[T>C]CA",
    "CA[T>C]CC",
    "CA[T>C]CG",
    "CA[T>C]CT",
    "CA[T>C]GA",
    "CA[T>C]GC",
    "CA[T>C]GG",
    "CA[T>C]GT",
    "CA[T>C]TA",
    "CA[T>C]TC",
    "CA[T>C]TG",
    "CA[T>C]TT",
    "CC[T>C]AA",
    "CC[T>C]AC",
    "CC[T>C]AG",
    "CC[T>C]AT",
    "CC[T>C]CA",
    "CC[T>C]CC",
    "CC[T>C]CG",
    "CC[T>C]CT",
    "CC[T>C]GA",
    "CC[T>C]GC",
    "CC[T>C]GG",
    "CC[T>C]GT",
    "CC[T>C]TA",
    "CC[T>C]TC",
    "CC[T>C]TG",
    "CC[T>C]TT",
    "CG[T>C]AA",
    "CG[T>C]AC",
    "CG[T>C]AG",
    "CG[T>C]AT",
    "CG[T>C]CA",    
    "CG[T>C]CC",
    "CG[T>C]CG",
    "CG[T>C]CT",
    "CG[T>C]GA",
    "CG[T>C]GC",
    "CG[T>C]GG",
    "CG[T>C]GT",
    "CG[T>C]TA",
    "CG[T>C]TC",
    "CG[T>C]TG",
    "CG[T>C]TT",
    "CT[T>C]AA",
    "CT[T>C]AC",
    "CT[T>C]AG",
    "CT[T>C]AT",
    "CT[T>C]CA",
    "CT[T>C]CC",
    "CT[T>C]CG",
    "CT[T>C]CT",
    "CT[T>C]GA",
    "CT[T>C]GC",
    "CT[T>C]GG",
    "CT[T>C]GT",
    "CT[T>C]TA",
    "CT[T>C]TC",
    "CT[T>C]TG",
    "CT[T>C]TT",
    "GA[T>C]AA",
    "GA[T>C]AC",
    "GA[T>C]AG",
    "GA[T>C]AT",
    "GA[T>C]CA",
    "GA[T>C]CC",
    "GA[T>C]CG",
    "GA[T>C]CT",
    "GA[T>C]GA",
    "GA[T>C]GC",
    "GA[T>C]GG",
    "GA[T>C]GT",
    "GA[T>C]TA",
    "GA[T>C]TC",
    "GA[T>C]TG",
    "GA[T>C]TT",
    "GC[T>C]AA",
    "GC[T>C]AC",
    "GC[T>C]AG",
    "GC[T>C]AT",
    "GC[T>C]CA",
    "GC[T>C]CC",
    "GC[T>C]CG",
    "GC[T>C]CT",
    "GC[T>C]GA",
    "GC[T>C]GC",
    "GC[T>C]GG",
    "GC[T>C]GT",
    "GC[T>C]TA",
    "GC[T>C]TC",
    "GC[T>C]TG",
    "GC[T>C]TT",    
    "GG[T>C]AA",
    "GG[T>C]AC",
    "GG[T>C]AG",
    "GG[T>C]AT",
    "GG[T>C]CA",
    "GG[T>C]CC",
    "GG[T>C]CG",
    "GG[T>C]CT",
    "GG[T>C]GA",
    "GG[T>C]GC",
    "GG[T>C]GG",
    "GG[T>C]GT",
    "GG[T>C]TA",
    "GG[T>C]TC",
    "GG[T>C]TG",
    "GG[T>C]TT",
    "GT[T>C]AA",
    "GT[T>C]AC",
    "GT[T>C]AG",
    "GT[T>C]AT",
    "GT[T>C]CA",
    "GT[T>C]CC",
    "GT[T>C]CG",
    "GT[T>C]CT",
    "GT[T>C]GA",
    "GT[T>C]GC",
    "GT[T>C]GG",
    "GT[T>C]GT",
    "GT[T>C]TA",
    "GT[T>C]TC",
    "GT[T>C]TG",
    "GT[T>C]TT",
    "TA[T>C]AA",
    "TA[T>C]AC",
    "TA[T>C]AG",
    "TA[T>C]AT",
    "TA[T>C]CA",
    "TA[T>C]CC",
    "TA[T>C]CG",
    "TA[T>C]CT",
    "TA[T>C]GA",
    "TA[T>C]GC",
    "TA[T>C]GG",
    "TA[T>C]GT",
    "TA[T>C]TA",
    "TA[T>C]TC",
    "TA[T>C]TG",
    "TA[T>C]TT",
    "TC[T>C]AA",
    "TC[T>C]AC",
    "TC[T>C]AG",
    "TC[T>C]AT",
    "TC[T>C]CA",
    "TC[T>C]CC",
    "TC[T>C]CG",
    "TC[T>C]CT",
    "TC[T>C]GA",
    "TC[T>C]GC",
    "TC[T>C]GG",
    "TC[T>C]GT",
    "TC[T>C]TA",
    "TC[T>C]TC",
    "TC[T>C]TG",
    "TC[T>C]TT",
    "TG[T>C]AA",
    "TG[T>C]AC",
    "TG[T>C]AG",
    "TG[T>C]AT",
    "TG[T>C]CA",
    "TG[T>C]CC",
    "TG[T>C]CG",
    "TG[T>C]CT",
    "TG[T>C]GA",
    "TG[T>C]GC",
    "TG[T>C]GG",
    "TG[T>C]GT",
    "TG[T>C]TA",
    "TG[T>C]TC",
    "TG[T>C]TG",
    "TG[T>C]TT",
    "TT[T>C]AA",
    "TT[T>C]AC",
    "TT[T>C]AG",
    "TT[T>C]AT",
    "TT[T>C]CA",
    "TT[T>C]CC",
    "TT[T>C]CG",
    "TT[T>C]CT",
    "TT[T>C]GA",
    "TT[T>C]GC",
    "TT[T>C]GG",
    "TT[T>C]GT",
    "TT[T>C]TA",
    "TT[T>C]TC",
    "TT[T>C]TG",
    "TT[T>C]TT",    
    "AA[T>G]AA",
    "AA[T>G]AC",
    "AA[T>G]AG",
    "AA[T>G]AT",
    "AA[T>G]CA",
    "AA[T>G]CC",
    "AA[T>G]CG",
    "AA[T>G]CT",
    "AA[T>G]GA",
    "AA[T>G]GC",
    "AA[T>G]GG",
    "AA[T>G]GT",
    "AA[T>G]TA",
    "AA[T>G]TC",
    "AA[T>G]TG",
    "AA[T>G]TT",
    "AC[T>G]AA",
    "AC[T>G]AC",
    "AC[T>G]AG",
    "AC[T>G]AT",
    "AC[T>G]CA",
    "AC[T>G]CC",
    "AC[T>G]CG",
    "AC[T>G]CT",
    "AC[T>G]GA",
    "AC[T>G]GC",
    "AC[T>G]GG",
    "AC[T>G]GT",
    "AC[T>G]TA",
    "AC[T>G]TC",
    "AC[T>G]TG",
    "AC[T>G]TT",
    "AG[T>G]AA",
    "AG[T>G]AC",
    "AG[T>G]AG",
    "AG[T>G]AT",
    "AG[T>G]CA",
    "AG[T>G]CC",
    "AG[T>G]CG",
    "AG[T>G]CT",
    "AG[T>G]GA",
    "AG[T>G]GC",
    "AG[T>G]GG",
    "AG[T>G]GT",
    "AG[T>G]TA",
    "AG[T>G]TC",
    "AG[T>G]TG",
    "AG[T>G]TT",
    "AT[T>G]AA",
    "AT[T>G]AC",
    "AT[T>G]AG",
    "AT[T>G]AT",
    "AT[T>G]CA",
    "AT[T>G]CC",
    "AT[T>G]CG",
    "AT[T>G]CT",
    "AT[T>G]GA",
    "AT[T>G]GC",
    "AT[T>G]GG",
    "AT[T>G]GT",
    "AT[T>G]TA",
    "AT[T>G]TC",
    "AT[T>G]TG",
    "AT[T>G]TT",
    "CA[T>G]AA",
    "CA[T>G]AC",
    "CA[T>G]AG",
    "CA[T>G]AT",
    "CA[T>G]CA",
    "CA[T>G]CC",
    "CA[T>G]CG",
    "CA[T>G]CT",
    "CA[T>G]GA",
    "CA[T>G]GC",
    "CA[T>G]GG",
    "CA[T>G]GT",
    "CA[T>G]TA",
    "CA[T>G]TC",
    "CA[T>G]TG",
    "CA[T>G]TT",
    "CC[T>G]AA",
    "CC[T>G]AC",
    "CC[T>G]AG",
    "CC[T>G]AT",
    "CC[T>G]CA",
    "CC[T>G]CC",
    "CC[T>G]CG",
    "CC[T>G]CT",
    "CC[T>G]GA",
    "CC[T>G]GC",
    "CC[T>G]GG",
    "CC[T>G]GT",
    "CC[T>G]TA",
    "CC[T>G]TC",
    "CC[T>G]TG",
    "CC[T>G]TT",
    "CG[T>G]AA",
    "CG[T>G]AC",
    "CG[T>G]AG",
    "CG[T>G]AT",
    "CG[T>G]CA",
    "CG[T>G]CC",
    "CG[T>G]CG",
    "CG[T>G]CT",
    "CG[T>G]GA",
    "CG[T>G]GC",
    "CG[T>G]GG",
    "CG[T>G]GT",
    "CG[T>G]TA",
    "CG[T>G]TC",
    "CG[T>G]TG",
    "CG[T>G]TT",
    "CT[T>G]AA",
    "CT[T>G]AC",
    "CT[T>G]AG",
    "CT[T>G]AT",
    "CT[T>G]CA",
    "CT[T>G]CC",
    "CT[T>G]CG",
    "CT[T>G]CT",
    "CT[T>G]GA",
    "CT[T>G]GC",
    "CT[T>G]GG",
    "CT[T>G]GT",
    "CT[T>G]TA",
    "CT[T>G]TC",
    "CT[T>G]TG",
    "CT[T>G]TT",
    "GA[T>G]AA",
    "GA[T>G]AC",
    "GA[T>G]AG",
    "GA[T>G]AT",
    "GA[T>G]CA",
    "GA[T>G]CC",
    "GA[T>G]CG",
    "GA[T>G]CT",
    "GA[T>G]GA",
    "GA[T>G]GC",
    "GA[T>G]GG",
    "GA[T>G]GT",
    "GA[T>G]TA",
    "GA[T>G]TC",
    "GA[T>G]TG",
    "GA[T>G]TT",
    "GC[T>G]AA",
    "GC[T>G]AC",
    "GC[T>G]AG",
    "GC[T>G]AT",
    "GC[T>G]CA",
    "GC[T>G]CC",
    "GC[T>G]CG",
    "GC[T>G]CT",
    "GC[T>G]GA",
    "GC[T>G]GC",
    "GC[T>G]GG",
    "GC[T>G]GT",
    "GC[T>G]TA",
    "GC[T>G]TC",
    "GC[T>G]TG",
    "GC[T>G]TT",
    "GG[T>G]AA",
    "GG[T>G]AC",
    "GG[T>G]AG",
    "GG[T>G]AT",
    "GG[T>G]CA",
    "GG[T>G]CC",
    "GG[T>G]CG",
    "GG[T>G]CT",
    "GG[T>G]GA",
    "GG[T>G]GC",
    "GG[T>G]GG",
    "GG[T>G]GT",
    "GG[T>G]TA",
    "GG[T>G]TC",
    "GG[T>G]TG",
    "GG[T>G]TT",
    "GT[T>G]AA",
    "GT[T>G]AC",
    "GT[T>G]AG",
    "GT[T>G]AT",
    "GT[T>G]CA",
    "GT[T>G]CC",
    "GT[T>G]CG",
    "GT[T>G]CT",
    "GT[T>G]GA",
    "GT[T>G]GC",
    "GT[T>G]GG",
    "GT[T>G]GT",
    "GT[T>G]TA",
    "GT[T>G]TC",
    "GT[T>G]TG",
    "GT[T>G]TT",
    "TA[T>G]AA",
    "TA[T>G]AC",
    "TA[T>G]AG",
    "TA[T>G]AT",
    "TA[T>G]CA",
    "TA[T>G]CC",
    "TA[T>G]CG",
    "TA[T>G]CT",
    "TA[T>G]GA",
    "TA[T>G]GC",
    "TA[T>G]GG",
    "TA[T>G]GT",
    "TA[T>G]TA",
    "TA[T>G]TC",
    "TA[T>G]TG",
    "TA[T>G]TT",
    "TC[T>G]AA",
    "TC[T>G]AC",
    "TC[T>G]AG",
    "TC[T>G]AT",
    "TC[T>G]CA",
    "TC[T>G]CC",
    "TC[T>G]CG",
    "TC[T>G]CT",
    "TC[T>G]GA",
    "TC[T>G]GC",
    "TC[T>G]GG",
    "TC[T>G]GT",
    "TC[T>G]TA",
    "TC[T>G]TC",
    "TC[T>G]TG",
    "TC[T>G]TT",
    "TG[T>G]AA",
    "TG[T>G]AC",
    "TG[T>G]AG",
    "TG[T>G]AT",
    "TG[T>G]CA",
    "TG[T>G]CC",
    "TG[T>G]CG",
    "TG[T>G]CT",
    "TG[T>G]GA",
    "TG[T>G]GC",
    "TG[T>G]GG",
    "TG[T>G]GT",
    "TG[T>G]TA",
    "TG[T>G]TC",
    "TG[T>G]TG",
    "TG[T>G]TT",
    "TT[T>G]AA",
    "TT[T>G]AC",
    "TT[T>G]AG",
    "TT[T>G]AT",
    "TT[T>G]CA",
    "TT[T>G]CC",
    "TT[T>G]CG",
    "TT[T>G]CT",
    "TT[T>G]GA",
    "TT[T>G]GC",
    "TT[T>G]GG",
    "TT[T>G]GT",
    "TT[T>G]TA",
    "TT[T>G]TC",
    "TT[T>G]TG",
    "TT[T>G]TT",  
]


sbs96_to_sbs52 = {
    "A[C>A]A":"A[C>A]A",
    "A[C>A]C":"A[C>A]C",
    "A[C>A]G":"A[C>A]G",
    "A[C>A]T":"A[C>A]T",
    "C[C>A]A":"C[C>A]A",
    "C[C>A]C":"C[C>A]C",
    "C[C>A]G":"C[C>A]G",
    "C[C>A]T":"C[C>A]T",
    "G[C>A]A":"G[C>A]A",
    "G[C>A]C":"G[C>A]C",
    "G[C>A]G":"G[C>A]G",
    "G[C>A]T":"G[C>A]T",
    "T[C>A]A":"T[C>A]A",
    "T[C>A]C":"T[C>A]C",
    "T[C>A]G":"T[C>A]G",
    "T[C>A]T":"T[C>A]T",
    "T[T>G]T":"A[C>A]A",
    "G[T>G]T":"A[C>A]C",
    "C[T>G]T":"A[C>A]G",
    "A[T>G]T":"A[C>A]T",
    "T[T>G]G":"C[C>A]A",
    "G[T>G]G":"C[C>A]C",
    "C[T>G]G":"C[C>A]G",
    "A[T>G]G":"C[C>A]T",
    "T[T>G]C":"G[C>A]A",
    "G[T>G]C":"G[C>A]C",
    "C[T>G]C":"G[C>A]G",
    "A[T>G]C":"G[C>A]T",
    "T[T>G]A":"T[C>A]A",
    "G[T>G]A":"T[C>A]C",
    "C[T>G]A":"T[C>A]G",
    "A[T>G]A":"T[C>A]T",
    "A[C>T]A":"A[C>T]A",
    "A[C>T]C":"A[C>T]C",
    "A[C>T]G":"A[C>T]G",
    "A[C>T]T":"A[C>T]T",
    "C[C>T]A":"C[C>T]A",
    "C[C>T]C":"C[C>T]C",
    "C[C>T]G":"C[C>T]G",
    "C[C>T]T":"C[C>T]T",
    "G[C>T]A":"G[C>T]A",
    "G[C>T]C":"G[C>T]C",
    "G[C>T]G":"G[C>T]G",
    "G[C>T]T":"G[C>T]T",
    "T[C>T]A":"T[C>T]A",
    "T[C>T]C":"T[C>T]C",
    "T[C>T]G":"T[C>T]G",
    "T[C>T]T":"T[C>T]T",
    "A[T>C]A":"A[C>T]A",
    "A[T>C]C":"A[C>T]C",
    "A[T>C]G":"A[C>T]G",
    "A[T>C]T":"A[C>T]T",
    "C[T>C]A":"C[C>T]A",
    "C[T>C]C":"C[C>T]C",
    "C[T>C]G":"C[C>T]G",
    "C[T>C]T":"C[C>T]T",
    "G[T>C]A":"G[C>T]A",
    "G[T>C]C":"G[C>T]C",
    "G[T>C]G":"G[C>T]G",
    "G[T>C]T":"G[C>T]T",
    "T[T>C]A":"T[C>T]A",
    "T[T>C]C":"T[C>T]C",
    "T[T>C]G":"T[C>T]G",
    "T[T>C]T":"T[C>T]T",
    "A[C>G]A":"A[C>G]A",
    "A[C>G]C":"A[C>G]C",
    "A[C>G]G":"A[C>G]G",
    "A[C>G]T":"A[C>G]T",
    "C[C>G]A":"C[C>G]A",
    "C[C>G]C":"C[C>G]C",
    "C[C>G]G":"C[C>G]G",
    "C[C>G]T":"A[C>G]G",
    "G[C>G]A":"G[C>G]A",
    "G[C>G]C":"G[C>G]C",
    "G[C>G]G":"C[C>G]C",
    "G[C>G]T":"A[C>G]C",
    "T[C>G]A":"T[C>G]A",
    "T[C>G]C":"G[C>G]A",
    "T[C>G]G":"C[C>G]A",
    "T[C>G]T":"A[C>G]A",
    "A[T>A]A":"A[T>A]A",
    "A[T>A]C":"A[T>A]C",
    "A[T>A]G":"A[T>A]G",
    "A[T>A]T":"A[T>A]T",
    "C[T>A]A":"C[T>A]A",
    "C[T>A]C":"C[T>A]C",
    "C[T>A]G":"C[T>A]G",
    "C[T>A]T":"A[T>A]G",
    "G[T>A]A":"G[T>A]A",
    "G[T>A]C":"G[T>A]C",
    "G[T>A]G":"C[T>A]C",
    "G[T>A]T":"A[T>A]C",
    "T[T>A]A":"T[T>A]A",
    "T[T>A]C":"G[T>A]A",
    "T[T>A]G":"C[T>A]A",
    "T[T>A]T":"A[T>A]A"
}


sbs96_to_sub = {
    "A[C>A]A": "C>A",
    "A[C>A]T": "C>A",
    "A[C>A]G": "C>A",
    "A[C>A]C": "C>A",
    "T[C>A]A": "C>A",
    "T[C>A]T": "C>A",
    "T[C>A]G": "C>A",
    "T[C>A]C": "C>A",
    "G[C>A]A": "C>A",
    "G[C>A]T": "C>A",
    "G[C>A]G": "C>A",
    "G[C>A]C": "C>A",
    "C[C>A]A": "C>A",
    "C[C>A]T": "C>A",
    "C[C>A]G": "C>A",
    "C[C>A]C": "C>A",
    "A[C>G]A": "C>G",
    "A[C>G]T": "C>G",
    "A[C>G]G": "C>G",
    "A[C>G]C": "C>G",
    "T[C>G]A": "C>G",
    "T[C>G]T": "C>G",
    "T[C>G]G": "C>G",
    "T[C>G]C": "C>G",
    "G[C>G]A": "C>G",
    "G[C>G]T": "C>G",
    "G[C>G]G": "C>G",
    "G[C>G]C": "C>G",
    "C[C>G]A": "C>G",
    "C[C>G]T": "C>G",
    "C[C>G]G": "C>G",
    "C[C>G]C": "C>G",
    "A[C>T]A": "C>T",
    "A[C>T]T": "C>T",
    "A[C>T]G": "C>T",
    "A[C>T]C": "C>T",
    "T[C>T]A": "C>T",
    "T[C>T]T": "C>T",
    "T[C>T]G": "C>T",
    "T[C>T]C": "C>T",
    "G[C>T]A": "C>T",
    "G[C>T]T": "C>T",
    "G[C>T]G": "C>T",
    "G[C>T]C": "C>T",
    "C[C>T]A": "C>T",
    "C[C>T]T": "C>T",
    "C[C>T]G": "C>T",
    "C[C>T]C": "C>T",
    "A[T>A]A": "T>A",
    "A[T>A]T": "T>A",
    "A[T>A]G": "T>A",
    "A[T>A]C": "T>A",
    "T[T>A]A": "T>A",
    "T[T>A]T": "T>A",
    "T[T>A]G": "T>A",
    "T[T>A]C": "T>A",
    "G[T>A]A": "T>A",
    "G[T>A]T": "T>A",
    "G[T>A]G": "T>A",
    "G[T>A]C": "T>A",
    "C[T>A]A": "T>A",
    "C[T>A]T": "T>A",
    "C[T>A]G": "T>A",
    "C[T>A]C": "T>A",
    "A[T>C]A": "T>C",
    "A[T>C]T": "T>C",
    "A[T>C]G": "T>C",
    "A[T>C]C": "T>C",
    "T[T>C]A": "T>C",
    "T[T>C]T": "T>C",
    "T[T>C]G": "T>C",
    "T[T>C]C": "T>C",
    "G[T>C]A": "T>C",
    "G[T>C]T": "T>C",
    "G[T>C]G": "T>C",
    "G[T>C]C": "T>C",
    "C[T>C]A": "T>C",
    "C[T>C]T": "T>C",
    "C[T>C]G": "T>C",
    "C[T>C]C": "T>C",
    "A[T>G]A": "T>G",
    "A[T>G]T": "T>G",
    "A[T>G]G": "T>G",
    "A[T>G]C": "T>G",
    "T[T>G]A": "T>G",
    "T[T>G]T": "T>G",
    "T[T>G]G": "T>G",
    "T[T>G]C": "T>G",
    "G[T>G]A": "T>G",
    "G[T>G]T": "T>G",
    "G[T>G]G": "T>G",
    "G[T>G]C": "T>G",
    "C[T>G]A": "T>G",
    "C[T>G]T": "T>G",
    "C[T>G]G": "T>G",
    "C[T>G]C": "T>G",
}


sbs96_to_tri = {
    "A[C>A]A": "ACA",
    "A[C>A]T": "ACT",
    "A[C>A]G": "ACG",
    "A[C>A]C": "ACC",
    "T[C>A]A": "TCA",
    "T[C>A]T": "TCT",
    "T[C>A]G": "TCG",
    "T[C>A]C": "TCC",
    "G[C>A]A": "GCA",
    "G[C>A]T": "GCT",
    "G[C>A]G": "GCG",
    "G[C>A]C": "GCC",
    "C[C>A]A": "CCA",
    "C[C>A]T": "CCT",
    "C[C>A]G": "CCG",
    "C[C>A]C": "CCC",
    "A[C>G]A": "ACA",
    "A[C>G]T": "ACT",
    "A[C>G]G": "ACG",
    "A[C>G]C": "ACC",
    "T[C>G]A": "TCA",
    "T[C>G]T": "TCT",
    "T[C>G]G": "TCG",
    "T[C>G]C": "TCC",
    "G[C>G]A": "GCA",
    "G[C>G]T": "GCT",
    "G[C>G]G": "GCG",
    "G[C>G]C": "GCC",
    "C[C>G]A": "CCA",
    "C[C>G]T": "CCT",
    "C[C>G]G": "CCG",
    "C[C>G]C": "CCC",
    "A[C>T]A": "ACA",
    "A[C>T]T": "ACT",
    "A[C>T]G": "ACG",
    "A[C>T]C": "ACC",
    "T[C>T]A": "TCA",
    "T[C>T]T": "TCT",
    "T[C>T]G": "TCG",
    "T[C>T]C": "TCC",
    "G[C>T]A": "GCA",
    "G[C>T]T": "GCT",
    "G[C>T]G": "GCG",
    "G[C>T]C": "GCC",
    "C[C>T]A": "CCA",
    "C[C>T]T": "CCT",
    "C[C>T]G": "CCG",
    "C[C>T]C": "CCC",
    "A[T>A]A": "ATA",
    "A[T>A]T": "ATT",
    "A[T>A]G": "ATG",
    "A[T>A]C": "ATC",
    "T[T>A]A": "TTA",
    "T[T>A]T": "TTT",
    "T[T>A]G": "TTG",
    "T[T>A]C": "TTC",
    "G[T>A]A": "GTA",
    "G[T>A]T": "GTT",
    "G[T>A]G": "GTG",
    "G[T>A]C": "GTC",
    "C[T>A]A": "CTA",
    "C[T>A]T": "CTT",
    "C[T>A]G": "CTG",
    "C[T>A]C": "CTC",
    "A[T>C]A": "ATA",
    "A[T>C]T": "ATT",
    "A[T>C]G": "ATG",
    "A[T>C]C": "ATC",
    "T[T>C]A": "TTA",
    "T[T>C]T": "TTT",
    "T[T>C]G": "TTG",
    "T[T>C]C": "TTC",
    "G[T>C]A": "GTA",
    "G[T>C]T": "GTT",
    "G[T>C]G": "GTG",
    "G[T>C]C": "GTC",
    "C[T>C]A": "CTA",
    "C[T>C]T": "CTT",
    "C[T>C]G": "CTG",
    "C[T>C]C": "CTC",
    "A[T>G]A": "ATA",
    "A[T>G]T": "ATT",
    "A[T>G]G": "ATG",
    "A[T>G]C": "ATC",
    "T[T>G]A": "TTA",
    "T[T>G]T": "TTT",
    "T[T>G]G": "TTG",
    "T[T>G]C": "TTC",
    "G[T>G]A": "GTA",
    "G[T>G]T": "GTT",
    "G[T>G]G": "GTG",
    "G[T>G]C": "GTC",
    "C[T>G]A": "CTA",
    "C[T>G]T": "CTT",
    "C[T>G]G": "CTG",
    "C[T>G]C": "CTC",
}

def get_sbs52(
    sbs96: str
) -> str:

    sbs52 = sbs96_to_sbs52[sbs96]
    return sbs52


def get_sbs96(
    chrom: str,
    pos: str,
    ref: str,
    alt: str,
    refseq,
) -> str:
    if ref in purine:
        upstream = purine2pyrimidine.get(refseq[chrom][pos + 1], "N")
        downstream = purine2pyrimidine.get(refseq[chrom][pos - 1], "N")
        sbs96 = "{}[{}>{}]{}".format(
            upstream,
            purine2pyrimidine.get(ref, "N"),
            purine2pyrimidine.get(alt, "N"),
            downstream,
        )
    else:
        upstream = refseq[chrom][pos - 1]
        downstream = refseq[chrom][pos + 1]
        sbs96 = "{}[{}>{}]{}".format(upstream, ref, alt, downstream)
    return sbs96


def get_sbs1536(
    chrom: str,
    pos: str,
    ref: str,
    alt: str,
    refseq,
) -> str:

    if ref in purine:
        penta_upstream = refseq[chrom][pos + 2]
        upstream = refseq[chrom][pos + 1]
        downstream = refseq[chrom][pos - 1]
        penta_downstream = refseq[chrom][pos - 2]
        sbs1536 = "{}{}[{}>{}]{}{}".format(
            purine2pyrimidine.get(penta_upstream, "N"),
            purine2pyrimidine.get(upstream, "N"),
            purine2pyrimidine.get(ref, "N"),
            purine2pyrimidine.get(alt, "N"),
            purine2pyrimidine.get(downstream, "N"),
            purine2pyrimidine.get(penta_downstream, "N")
        )
    else:
        penta_upstream = refseq[chrom][pos - 2]
        upstream = refseq[chrom][pos - 1]
        downstream = refseq[chrom][pos + 1]
        penta_downstream = refseq[chrom][pos + 2]
        sbs1536 = "{}{}[{}>{}]{}{}".format(
            penta_upstream,
            upstream, 
            ref, 
            alt, 
            downstream,
            penta_downstream
        )
    return sbs1536


def load_sbs96_counts(
    vcf_file: str, ref_file: str, chrom_lst: List[str]
) -> Dict[str, int]:

    tname_lst = []
    tname2sbs2counts = defaultdict()
    refseq = pyfastx.Fasta(ref_file)
    if vcf_file.endswith(".vcf"):
        for line in open(vcf_file).readlines():
            if line.startswith("##"):
                if line.startswith("##contig"):
                    tname = line.strip().replace("##contig=<ID=", "").split(",")[0]
                    tname_lst.append(tname)
                continue
            elif line.startswith("#CHROM"):
                for tname in tname_lst:
                    tname2sbs2counts[tname] = defaultdict(lambda: 0)
                continue
            v = himut.vcflib.VCF(line)
            if v.is_snp and v.is_pass and v.is_biallelic:
                sbs96 = get_sbs96(v.chrom, int(v.pos) - 1, v.ref, v.alt, refseq)
                tname2sbs2counts[v.chrom][sbs96] += 1
    elif vcf_file.endswith(".vcf.bgz"):
        for line in cyvcf2.VCF(vcf_file).raw_header.split("\n"):
            if line.startswith("##"):
                if line.startswith("##contig"):
                    tname = line.replace("##contig=<ID=", "").split(",")[0]
                    tname_lst.append(tname)
                continue
            elif line.startswith("#CHROM"):
                for tname in tname_lst:
                    tname2sbs2counts[tname] = defaultdict(lambda: 0)
        for i in cyvcf2.VCF(vcf_file):
            v = himut.vcflib.VCF(str(i))
            if v.is_snp and v.is_pass and v.is_biallelic:
                sbs96 = get_sbs96(v.chrom, int(v.pos) - 1, v.ref, v.alt, refseq)
                tname2sbs2counts[v.chrom][sbs96] += 1

    sbs2count = defaultdict(lambda: 0)
    for chrom in chrom_lst:
        for sbs, count in tname2sbs2counts[chrom].items():
            sbs2count[sbs] += count
    return sbs2count


def load_sbs1536_counts(
    vcf_file: str, ref_file: str, chrom_lst: List[str]
) -> Dict[str, int]:

    tname_lst = []
    tname2sbs2counts = defaultdict()
    refseq = pyfastx.Fasta(ref_file)
    if vcf_file.endswith(".vcf"):
        for line in open(vcf_file).readlines():
            if line.startswith("##"):
                if line.startswith("##contig"):
                    tname = line.strip().replace("##contig=<ID=", "").split(",")[0]
                    tname_lst.append(tname)
                continue
            elif line.startswith("#CHROM"):
                for tname in tname_lst:
                    tname2sbs2counts[tname] = defaultdict(lambda: 0)
                continue
            v = himut.vcflib.VCF(line)
            if v.is_snp and v.is_pass and v.is_biallelic:
                sbs1536 = get_sbs1536(v.chrom, int(v.pos) - 1, v.ref, v.alt, refseq)
                tname2sbs2counts[v.chrom][sbs1536] += 1
    elif vcf_file.endswith(".vcf.bgz"):
        for line in cyvcf2.VCF(vcf_file).raw_header.split("\n"):
            if line.startswith("##"):
                if line.startswith("##contig"):
                    tname = line.replace("##contig=<ID=", "").split(",")[0]
                    tname_lst.append(tname)
                continue
            elif line.startswith("#CHROM"):
                for tname in tname_lst:
                    tname2sbs2counts[tname] = defaultdict(lambda: 0)
        for i in cyvcf2.VCF(vcf_file):
            v = himut.vcflib.VCF(str(i))
            if v.is_snp and v.is_pass and v.is_biallelic:
                sbs1536 = get_sbs1536(v.chrom, int(v.pos) - 1, v.ref, v.alt, refseq)
                tname2sbs2counts[v.chrom][sbs1536] += 1

    sbs2count = defaultdict(lambda: 0)
    for chrom in chrom_lst:
        for sbs, count in tname2sbs2counts[chrom].items():
            sbs2count[sbs] += count
    return sbs2count


def dump_sbs52_counts(
    vcf_file: str, 
    ref_file: str, 
    region: str, 
    region_list: str, 
    tname2tsize: Dict[str, int],
    out_file: str
) -> None:

    sbs52_counts = {sbs52: 0 for sbs52 in sbs52_lst}
    chrom_lst, _ = himut.util.load_loci(region, region_list, tname2tsize)
    sbs96_counts = load_sbs96_counts(vcf_file, ref_file, chrom_lst)
    for sbs96, counts in sbs96_counts.items():
        sbs52 = sbs96_to_sbs52[sbs96]
        sbs52_counts[sbs52] += counts

    o = open(out_file, "w")
    o.write("{}\t{}\t{}\t{}\n".format("sub", "tri", "sbs52", "counts"))
    for sbs52 in sbs52_lst:
        o.write(
            "{}\t{}\t{}\t{}\n".format(sbs96_to_sub[sbs52], sbs96_to_tri[sbs52], sbs52, sbs52_counts[sbs52])
        )
    o.close()


def dump_sbs96_counts(
    vcf_file: str, 
    ref_file: str, 
    region: str, 
    region_list: str, 
    tname2tsize: Dict[str, int],
    out_file: str
) -> None:

    chrom_lst, _ = himut.util.load_loci(region, region_list, tname2tsize)
    sbs96_counts = load_sbs96_counts(vcf_file, ref_file, chrom_lst)
    o = open(out_file, "w")
    o.write("{}\t{}\t{}\t{}\n".format("sub", "tri", "sbs96", "counts"))
    for sbs96 in sbs96_lst:
        o.write(
            "{}\t{}\t{}\t{}\n".format(sbs96_to_sub[sbs96], sbs96_to_tri[sbs96], sbs96, sbs96_counts[sbs96])
        )
    o.close()


def dump_sbs1536_counts(
    vcf_file: str, 
    ref_file: str, 
    region: str, 
    region_list: str, 
    tname2tsize: Dict[str, int],
    out_file: str
) -> Dict[str, int]:


    tri_sub_sum_hsh = defaultdict(lambda: 0) 
    chrom_lst, _ = himut.util.load_loci(region, region_list, tname2tsize)
    sbs1536_counts = load_sbs1536_counts(vcf_file, ref_file, chrom_lst)
    for sbs1536 in sbs1536_lst:
        uu, u, _, ref, _, alt, _, d, dd = list(sbs1536)
        sub = "{}>{}".format(ref, alt)
        tri = "{}{}{}".format(u, ref, d)
        tri_sub_count = sbs1536_counts[sbs1536]
        tri_sub_sum_hsh["{},{}".format(tri, sub)] += tri_sub_count

    o = open(out_file, "w")
    o.write("{}\t{}\t{}\t{}\t{}\t{}\n".format("sub", "tri", "penta", "sbs1536", "counts", "proportion"))
    for sbs1536 in sbs1536_lst:
        uu, u, _, ref, _, alt, _, d, dd = list(sbs1536)
        sub = "{}>{}".format(ref, alt)
        tri = "{}{}{}".format(u, ref, d)
        penta = "{}---{}".format(uu, dd)
        tri_sub_count = sbs1536_counts[sbs1536]
        tri_sub_sum = tri_sub_sum_hsh["{},{}".format(tri, sub)]
        tri_sub_proportion = tri_sub_count/float(tri_sub_sum) 
        o.write(
            "{}\t{}\t{}\t{}\t{}\t{}\n".format(sub, tri, penta, sbs1536, tri_sub_count, tri_sub_proportion)
        )
    o.close()


def dump_sbs96_plt(infile: str, sample: str, outfile: str) -> None:

    if sample is None:
        print(
            "Sample cannot be None\nBAM or VCF file might be missing sample information"
        )
        himut.util.exit()

    df = pd.read_csv(infile, sep="\t")
    plot = (
        ggplot(df, aes(x="tri", y="counts", fill="sub"))
        + geom_bar(stat="identity")
        + theme_bw()
        + facet_grid(". ~ sub", scales="free")
        + scale_fill_manual(
            values=("#98D7EC", "#212121", "#FF003A", "#A6A6A6", "#83A603", "#F5ABCC")
        )
        + labs(x="\nTrinucleotide Context\n", y="\nCounts\n")
        + ggtitle("\n{}\n".format(sample))
        + theme(
            text=element_text(size=10),
            legend_title=element_blank(),
            axis_text_x=element_text(
                family="monospace", angle=90, ha="center"
            ),
        )
    )
    plot.save(outfile, width=22, height=12)


def dump_norm_sbs96_plt(infile: str, sample: str, outfile: str) -> None:

    if sample is None:
        print(
            "Sample cannot be None\nBAM or VCF file might be missing sample information"
        )
        himut.util.exit()

    df = pd.read_csv(infile, sep="\t", comment="#")
    plot = (
        ggplot(df, aes(x="tri", y="normcounts", fill="sub"))
        + geom_bar(stat="identity")
        + theme_bw()
        + facet_grid(". ~ sub", scales="free")
        + scale_fill_manual(
            values=("#98D7EC", "#212121", "#FF003A", "#A6A6A6", "#83A603", "#F5ABCC")
        )
        + labs(x="\nTrinucleotide Context\n", y="\nCounts\n")
        + ggtitle("\n{}\n".format(sample))
        + theme(
            text=element_text(size=10),
            legend_title=element_blank(),
            axis_text_x=element_text(
                family="monospace", angle=90, ha="center"
            ),
        )
    )
    plot.save(outfile, width=22, height=12)


def dump_sbs1536_plt(infile: str, sample: str, outfile: str) -> None:

    if sample is None:
        print(
            "Sample cannot be None\nBAM or VCF file might be missing sample information"
        )
        himut.util.exit()

    df = pd.read_csv(infile, sep="\t")
    plot = (
        ggplot(df, aes(x="tri", y="counts", fill="sub"))
        + geom_bar(stat="identity")
        + theme_bw()
        + facet_grid(". ~ sub", scales="free")
        + scale_fill_manual(
            values=("#98D7EC", "#212121", "#FF003A", "#A6A6A6", "#83A603", "#F5ABCC")
        )
        + labs(x="\nTrinucleotide Context\n", y="\nCounts\n")
        + ggtitle("\n{}\n".format(sample))
        + theme(
            text=element_text(size=10),
            legend_title=element_blank(),
            axis_text_x=element_text(
                family="monospace", angle=90, ha="center"
            ),
        )
    )
    plot.save(outfile, width=22, height=12)


def get_trifreq(tri2count: Dict[str, int]) -> Dict[str, float]:
 
    tri2freq = {}
    tri_sum = sum(tri2count.values())
    for tri, count in tri2count.items():
        if tri_sum == 0:
            tri2freq[tri] = 0
        else:
            tri2freq[tri] = count/float(tri_sum)
    return tri2freq


def get_trifreq_ratio(
    ref_tri2count: Dict[str, int],
    ccs_tri2count: Dict[str, int],
) -> Dict[str, float]:

    trifreq_ratio = {}
    ref_tri2freq = get_trifreq(ref_tri2count)
    ccs_tri2freq = get_trifreq(ccs_tri2count)
    for tri in tri_lst:
        ref_tri_freq = ref_tri2freq[tri] 
        ccs_tri_freq = ccs_tri2freq[tri]
        if ccs_tri_freq == 0:
            trifreq_ratio[tri] = 0
        else: 
            trifreq_ratio[tri] = ref_tri_freq/float(ccs_tri_freq)
    return trifreq_ratio


def get_cumsum_tricounts(
    chrom2tri2count: Dict[str, Dict[str, int]]
) -> Tuple[int, Dict[str, int]]:

    tri2count = defaultdict(lambda: 0)
    for chrom in chrom2tri2count:
        for tri in tri_lst:
            tri_count = chrom2tri2count[chrom][tri]
            tri2count[tri] += tri_count
    return tri2count


def get_normcounts_cmdline(
    bam_file: str,
    ref_file: str,
    sbs_file: str,
    vcf_file: str,
    phased_vcf_file: str,
    min_qv,
    min_mapq: int,
    min_sequence_identity: float,
    min_gq: int,
    min_bq: int,
    min_trim: float,
    mismatch_window: int,
    max_mismatch_count: int,
    min_ref_count: int,
    min_alt_count: int,
    min_hap_count: int,
    common_snps: str,
    panel_of_normals: str,
    somatic_snv_prior: float,
    germline_snv_prior: float,
    germline_indel_prior: float,
    threads: int,
    phase: bool,
    non_human_sample: bool,
    reference_sample: bool,
    out_file: str,
):

    param = "--min_qv {} --min_mapq {} --min_sequence_identity {} --min_gq {} --min_bq {} --min_ref_count {} --min_alt_count {} --min_hap_count {} --min_trim {} --mismatch_window {} --max_mismatch_count {} --somatic_snv_prior {} --germline_snv_prior {} --germline_indel_prior {} --threads {} -o {}".format(
        min_qv,
        min_mapq,
        min_sequence_identity,
        min_gq,
        min_bq,
        min_ref_count,
        min_alt_count,
        min_hap_count,
        min_trim,
        mismatch_window,
        max_mismatch_count,
        somatic_snv_prior,
        germline_snv_prior,
        germline_indel_prior,
        threads,
        out_file,
    )

    if phase and non_human_sample and reference_sample:
        cmdline = "##himut_command=himut normcounts -i {} --ref {} --sbs {} --vcf {} --phased_vcf {} {} --phase --non_human_sample --reference_sample".format(
            bam_file, ref_file, sbs_file, vcf_file, phased_vcf_file, param
        )
    elif phase and non_human_sample and not reference_sample:
        cmdline = "##himut_command=himut normcounts -i {} --ref {} --sbs {} --vcf {} --phased_vcf {} {} --phase --non_human_sample".format(
            bam_file, ref_file, sbs_file, vcf_file, phased_vcf_file, param
        )
    elif phase and not non_human_sample and not reference_sample:
        cmdline = "##himut_command=himut normcounts -i {} --ref {} --sbs {} --phased_vcf {} {} --common_snps {} --panel_of_normals {} --phase".format(
            bam_file,
            ref_file,
            sbs_file,
            phased_vcf_file,
            param,
            common_snps,
            panel_of_normals,
        )
    elif not phase and non_human_sample and reference_sample:
        cmdline = "##himut_command=himut normcounts -i {} --ref {} --sbs {} --vcf {} {} --non_human_sample --reference_sample".format(
            bam_file, ref_file, sbs_file, vcf_file, param
        )
    elif not phase and non_human_sample and not reference_sample:
        cmdline = "##himut_command=himut normcounts -i {} --ref {} --sbs {} --vcf {} {} --non_human_sample".format(
            bam_file, ref_file, sbs_file, vcf_file, param
        )
    elif not phase and not non_human_sample and not reference_sample:
        cmdline = "##himut_command=himut normcounts -i {} --ref {} --sbs {} {} --common_snps {} --panel_of_normals {}".format(
            bam_file,
            ref_file,
            sbs_file,
            param,
            common_snps,
            panel_of_normals,
        )
    return cmdline


def dump_normcounts(
    sbs96_counts: Dict[str, int],
    ref_tri2count: Dict[str, int],
    chrom2ref_callable_tri2count: Dict[str, int],
    chrom2ccs_callable_tri2count: Dict[str, int],
    cmdline: str,
    out_file: str,
) -> None:

    ref_callable_tri2count = get_cumsum_tricounts(chrom2ref_callable_tri2count)
    ccs_callable_tri2count = get_cumsum_tricounts(chrom2ccs_callable_tri2count)
    ref_tri2freq_ratio = get_trifreq_ratio(ref_callable_tri2count, ref_tri2count)
    callable_tri2freq_ratio = get_trifreq_ratio(
        ref_callable_tri2count, ccs_callable_tri2count
    )

    o = open(out_file, "w")
    o.write("{}\n".format(cmdline))
    o.write(
        "{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\n".format(
            "sub",
            "tri",
            "sbs96",
            "counts",
            "normcounts",
            "ref_tri_ratio",
            "ref_ccs_tri_ratio",
            "ref_tri_count",
            "ref_callable_tri_count",
            "ccs_callable_tri_count",
        )
    )

    for sbs96 in sbs96_lst:
        sub = sbs96_to_sub[sbs96]
        tri = sbs96_to_tri[sbs96]
        count = sbs96_counts[sbs96]
        ref_tri_count = ref_tri2count[tri]
        ref_trifreq_ratio = ref_tri2freq_ratio[tri]
        ref_callable_tricount = ref_callable_tri2count[tri]
        ccs_callable_tricount = ccs_callable_tri2count[tri]
        ref_ccs_trifreq_ratio = callable_tri2freq_ratio[tri]
        normcount = count * ref_trifreq_ratio * ref_ccs_trifreq_ratio
        o.write(
            "{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\n".format(
                sub,
                tri,
                sbs96,
                count,
                normcount,
                ref_trifreq_ratio,
                ref_ccs_trifreq_ratio,
                ref_tri_count,
                ref_callable_tricount,
                ccs_callable_tricount,
            )
        )
    o.close()


def load_ref_tricount(
    ref_file: str,
    tri_file: str,
    region_list: str,
    threads: int,
):

    tri2count = 0
    chrom_lst = [line.strip() for line in open(region_list).readlines()]
    if ref_file is None and tri_file is not None:
        tri2count = dict(line.strip().split() for line in open(tri_file).readlines())
    elif ref_file is not None and tri_file is None:
        refseq = pyfastx.Fasta(ref_file)
        tri2count = himut.reflib.get_genome_tricounts(refseq, chrom_lst, threads)
    elif ref_file is not None and tri_file is not None:
        tri2count = dict(line.strip().split() for line in open(tri_file).readlines())
    else:
        print("Please provide either --ref or --tri file")
        himut.util.exit()

    for tri, count in tri2count.items():
        tri2count[tri] = int(count)
    return tri2count


def get_burden_per_cell(
    infile,
    ref_file,
    tri_file,
    region_list,
    threads,
    outfile,
):

    tri2mut_rate = defaultdict(lambda: 0)
    ref_tri2count = load_ref_tricount(ref_file, tri_file, region_list, threads)
    for line in open(infile).readlines():
        if line.startswith("#"):
            continue
        elif line.startswith("sub"):
            continue
        (
            _sub,
            tri,
            _sbs,
            _count,
            normcounts,
            _ref_tri_ratio,
            _ref_ccs_tri_ratio,
            _ref_tri_count,
            _ref_callable_tri_count,
            ccs_callable_tri_count,
        ) = line.strip().split()
        tri2mut_rate[tri] += float(normcounts) / int(ccs_callable_tri_count)

    ref_tri2count = load_ref_tricount(ref_file, tri_file, region_list, threads)
    total_mut_count = sum([tri2mut_rate[tri] * ref_tri2count[tri] for tri in tri_lst])
    burden_per_cell = total_mut_count * 2
    o = open(outfile, "w")
    o.write("{}\n".format(burden_per_cell))
    o.close()


def dump_norm_log(
    chrom_lst: List[str],
    chrom2norm_log: Dict[str, List[int]],
):

    row_names = [
        "num_ccs",
        "num_bases",
        "num_unphased_bases",
        "num_het_bases",
        "num_hetalt_bases",
        "num_homalt_bases",
        "num_homref_bases",
        "num_uncallable_bases",
        "num_md_filtered_bases",
        "num_ab_filtered_bases",
        "num_low_gq_bases",
        "num_pon_filtered_bases",
        "num_pop_filtered_bases",
        "num_callable_bases",
    ]
    ncol = len(chrom_lst)
    nrow = len(row_names)
    dt = np.zeros((nrow, ncol))
    for i, chrom in enumerate(chrom_lst):
        for j, count in enumerate(chrom2norm_log[chrom]):
            dt[j][i] = count

    o = open("norm.log", "w")
    col_lst = chrom_lst + ["total"]
    o.write("{:30}{}\n".format("", "\t".join(col_lst)))
    for k in range(nrow):
        rsum = str(int(np.sum(dt[k])))
        rlst = [str(int(r)) for r in dt[k].tolist()] + [rsum]
        o.write("{:30}{}\n".format(row_names[k], "\t".join(rlst)))
    o.close()
